//	yf Net list reader base
//		2005.10.11 - 2005.10.16 Yoshaiki.Ueda
#include "wx/wx.h"
#include "yfNetListReaderBase.h"
#include <wx/wfstream.h>
#include <wx/txtstrm.h>

//	Tokenizer	//=============================================================
yfNetListTokenizer::yfNetListTokenizer()
	: m_currentPosition(0)
{
}

void yfNetListTokenizer::SetKeywords(const wxString& keywords)
{
	m_keywords = keywords;
}

void yfNetListTokenizer::SetString(const wxString& to_tokenize)
{
	m_tokenizer.SetString(to_tokenize);
	m_currentToken.Empty();
	m_currentPosition = 0;
}

bool yfNetListTokenizer::HasMoreTokens() const
{
	return m_tokenizer.HasMoreTokens() || (m_currentPosition<m_currentToken.Length());
}

wxString yfNetListTokenizer::GetNextToken()
{
	if( m_currentPosition>=m_currentToken.Length() ){
		m_currentToken = m_tokenizer.GetNextToken();
		m_currentPosition = 0;
	}
	wxString token;
	int c;
	if( m_currentPosition<m_currentToken.Length() ){
		c = m_currentToken[m_currentPosition];
		token.Append(wxUniChar(c));
		m_currentPosition++;
		if( m_keywords.Find(wxUniChar(c))>=0 ) return token;
		while( m_currentPosition<m_currentToken.Length() ){
			c = m_currentToken[m_currentPosition];
			if( m_keywords.Find(wxUniChar(c))>=0 ) break;
			token.Append(wxUniChar(c));
			m_currentPosition++;
		}
	}
	return token;
}

//	Net list reader	//=========================================================
yfNetListReader::yfNetListReader()
	: m_currentLine(0), m_netList(NULL), m_currentNet(NULL)
{
}

yfNetListReader::~yfNetListReader()
{
	//yfNetList::Net::NodeList::iterator node;
	//for(node=m_currentNet->m_nodeList.begin();	node!=m_currentNet->m_nodeList.end();node++){
	//	
	//}
	//if(m_currentNet)m_currentNet->m_nodeList.clear(); //20131222
	delete m_currentNet;
}

bool yfNetListReader::Read(const wxString& fileName, yfNetList& netList)
{
	//init
	m_currentLine = 0;
	//if(m_currentNet)m_currentNet->m_nodeList.clear();
	delete m_currentNet;
	m_currentNet = NULL;
	m_netList = &netList;
	Init();
	//read
	wxFileInputStream input(fileName);
	wxTextInputStream text(input);
	wxString line;
	while( !input.Eof() ){
		line = text.ReadLine();
		netList.m_sourceText.push_back(line);
		ParseLine(line);
		m_currentLine++;
	}
	//end of net
	EndOfNet();
	return true;
}

void yfNetListReader::ParseLine(const wxString& source)
{
	m_tokenizer.SetString(source);
	wxString token;
	while(m_tokenizer.HasMoreTokens()){
		token=m_tokenizer.GetNextToken();
		//??//if(token.IsEmpty()) continue;
		//for debug//wxMessageBox( wxString::Format(wxT("[%s]"), (const char*)token) );
		OnToken(token);
		m_lastToken = token;
	}
	OnTermLine();
}

void yfNetListReader::BeginOfNet(const wxString& name)
{
	EndOfNet();
	//begin of net
	m_currentNet = new yfNetList::Net;
	m_currentNet->m_beginLine = m_currentLine;
	m_currentNet->m_name = name;
}

void yfNetListReader::EndOfNet()
{
	if(m_currentNet){
		//end of net
		m_currentNet->m_nextLine = m_currentLine;
		m_netList->m_list.push_back(*m_currentNet);
		delete m_currentNet;	//20131222
		m_currentNet = NULL;
	}
}

void yfNetListReader::OnNode(const wxString& name)
{
	if(m_currentNet){
		//yfNetList::Net::Node* node = new yfNetList::Net::Node;
		//node->m_name = name;
		yfNetList::Net::Node node;
		node.m_name = name;
		m_currentNet->m_nodeList.push_back(node);
	}
}
